/*-
 * Copyright (c) 2006, 2007 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      Runner.cpp
 *
 * $CTPP$
 */

#include <CTPP2.hpp>

#include <FnDateFormat.hpp>

int main(int argc, char ** argv)
{
	using namespace CTPP;
	if (argc != 2)
	{
		fprintf(stderr, "usage: %s file.name\n", argv[0]);
		return EX_USAGE;
	}

	// Initialize Standard CTPP library
	SyscallFactory oSyscallFactory(100);
	// Load standard library
	STDLibInitializer::InitLibrary(oSyscallFactory);

	/*
	 * Register our function in factory
	 */
	CTPP::SyscallHandler * pFnDateFormat = new MY_NS::FnDateFormat;
	oSyscallFactory.RegisterHandler(pFnDateFormat);

	try
	{
		// Load file
		VMFileLoader oLoader(argv[1]);

		// Get program core
		const VMMemoryCore * pVMMemoryCore = oLoader.GetCore();

		// Parameters
		CDT oHash(CDT::HASH_VAL);

		// Output collector
		FileOutputCollector oOutputCollector(stdout);

		// Create virtual machine
		VM oVM(oSyscallFactory, 10240, 10240, 10240);
		// Initialize virtual machine
		oVM.Init(oOutputCollector, *pVMMemoryCore);

		// Run Program
		UINT_32 iIP = 0;
		oVM.Run(*pVMMemoryCore, iIP, oHash);
	}
	catch(IllegalOpcode         & e) { fprintf(stderr, "ERROR: Illegal opcode at 0x%08X\n", e.GetIP());                 }
	catch(InvalidSyscall        & e) { fprintf(stderr, "ERROR: Invalid syscall `%s` at 0x%08X\n", e.what(), e.GetIP()); }
	catch(CodeSegmentOverrun    & e) { fprintf(stderr, "ERROR: %s at 0x%08X\n", e.what(),  e.GetIP());                  }
	catch(StackOverflow         & e) { fprintf(stderr, "ERROR: Stack overflow at 0x%08X\n", e.GetIP());                 }
	catch(StackUnderflow        & e) { fprintf(stderr, "ERROR: Stack underflow at 0x%08X\n", e.GetIP());                }
	catch(ExecutionLimitReached & e) { fprintf(stderr, "ERROR: Execution limit of %d step(s) reached at 0x%08X\n", 10240, e.GetIP()); }
	catch(CDTTypeCastException  & e) { fprintf(stderr, "ERROR: Type Cast %s\n", e.what());                              }

	// Destroy standard library functions
	STDLibInitializer::DestroyLibrary(oSyscallFactory);

	oSyscallFactory.RemoveHandler("date_format");
	delete pFnDateFormat;

	// make valgrind happy
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);

return EX_OK;
}
// End.
