/*-
 * Copyright (c) 2006, 2007 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      hello.cpp
 *
 * $CTPP$
 */

#include <CTPP2.hpp>

#include <stdio.h>

using namespace CTPP;

int main(int argc, char ** argv)
{
	if (argc != 2) { fprintf(stdout, "usage: %s file.ct2\n", argv[0]); return EX_USAGE; }

	try
	{
		// Output data collector would write to STDOUT
		FileOutputCollector oOutputCollector(stdout);

		// Create syscalls factory
		SyscallFactory oSyscallFactory(100);
		// .... and load standard library
		STDLibInitializer::InitLibrary(oSyscallFactory);

		// Load program from file
		VMFileLoader oLoader(argv[1]);

		// Get program core
		const VMMemoryCore * pVMMemoryCore = oLoader.GetCore();

		// Our data
		CDT oHash;
		oHash["hello"] = "Hello, World!";

		// Create virtual machine
		VM oVM(oSyscallFactory);

		// Initialize machine
		oVM.Init(oOutputCollector, *pVMMemoryCore);

		// Run program
		UINT_32 iIP = 0;
		oVM.Run(*pVMMemoryCore, iIP, oHash);

		// Destroy standard library
		STDLibInitializer::DestroyLibrary(oSyscallFactory);
	}
	catch(CTPPException & e)  { fprintf(stderr, "ERROR: %s\n", e.what()); }

// All done
return EX_OK;
}
// End.

