/*-
 * Copyright (c) 2004 - 2010 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      CTPP2SymbolTableTest.cpp
 *
 * $CTPP$
 */

#include <CTPP2SymbolTable.hpp>

#include <stdio.h>

int main(void)
{
	using namespace CTPP;
	SymbolTable<INT_32> oIntTable;

	INT_32 iID = oIntTable.AddSymbol("a", 1);
	fprintf(stderr, "Add a(0) = %d; Ok\n", iID);

	iID = oIntTable.AddSymbol("b", 1);
	fprintf(stderr, "Add b(1) = %d; Ok\n", iID);

	iID = oIntTable.AddSymbol("c", 1);
	fprintf(stderr, "Add c(2) = %d; Ok\n", iID);

	iID = oIntTable.AddSymbol("a", 1);
	fprintf(stderr, "Add a(-1) = %d; Cannot insert duplicate!\n", iID);

	const SymbolTable<INT_32>::SymbolRecord<INT_32> * pRecord = oIntTable.GetSymbol("c");
	if (pRecord != NULL) { fprintf(stderr, "Get c(2) = %d - %d; Ok\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "Get c(2) = Error!\n"); }

	oIntTable.MarkScope();
	fprintf(stderr, "MarkScope\n");

	iID = oIntTable.AddSymbol("b", 1);
	fprintf(stderr, "    Add b(3) = %d; Ok. New symbol shadows previous declaration.\n", iID);

	oIntTable.MarkScope();
	fprintf(stderr, "    MarkScope\n");

	iID = oIntTable.AddSymbol("b", 1);
	fprintf(stderr, "        Add b(4) = %d; Ok\n", iID);

	iID = oIntTable.AddSymbol("z", 1);
	fprintf(stderr, "        Add z(4) = %d; Ok\n", iID);

	oIntTable.UnmarkScope();
	fprintf(stderr, "    UnmarkScope\n");

	iID = oIntTable.AddSymbol("c", 1);
	fprintf(stderr, "    Add c(4) = %d; Ok. New symbol shadows previous declaration.\n", iID);

	iID = oIntTable.AddSymbol("d", 1);
	fprintf(stderr, "    Add d(5) = %d; Ok.\n", iID);

	pRecord = oIntTable.GetSymbol("c");
	if (pRecord != NULL) { fprintf(stderr, "    Get c(4) = %d - %d; Ok. From current scope\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "    Get c(4) = ###\n"); }

	oIntTable.UnmarkScope();
	fprintf(stderr, "UnmarkScope\n");

	pRecord = oIntTable.GetSymbol("a");
	if (pRecord != NULL) { fprintf(stderr, "Get a(0) = %d - %d\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "Get a(0) = ###\n"); }

	pRecord = oIntTable.GetSymbol("b");
	if (pRecord != NULL) { fprintf(stderr, "Get b(1) = %d - %d\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "Get b(1) = ###\n"); }

	pRecord = oIntTable.GetSymbol("c");
	if (pRecord != NULL) { fprintf(stderr, "Get c(2) = %d - %d\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "Get c(2) = ###\n"); }

	pRecord = oIntTable.GetSymbol("d");
	if (pRecord != NULL) { fprintf(stderr, "Get d(-) = %d - %d\n", pRecord -> symbol_id, pRecord -> symbol_data); }
	else                 { fprintf(stderr, "Get d(-) = ###; Ok. Symbol does not exist\n"); }
}
